<?php

namespace Drupal\password_policy_prlp\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\password_policy_extras\Event\CheckValidationEvent;
use Drupal\password_policy_extras\Event\CheckVisibilityEvent;
use Drupal\password_policy_extras\EventSubscriber\PasswordPolicyExtrasEventSubscriber;
use Drupal\password_policy_extras\PasswordPolicyExtrasEvents;
use Drupal\prlp\Event\ResetPasswordUpdateEvent;
use Drupal\prlp\Event\ResetPasswordValidationEvent;
use Drupal\prlp\PrlpEvents;
use Drupal\user\Entity\User;

/**
 * Password Policy User Registration Password events handling.
 */
class PasswordPolicyPasswordResetLandingPageEventSubscriber extends PasswordPolicyExtrasEventSubscriber {

  use StringTranslationTrait;

  /**
   * User Registration Password configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $prlp_settings;

  /**
   * PasswordPolicyEventSubscriber constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The currently logged-in user.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The current route match.
   */
  public function __construct(ConfigFactoryInterface $config_factory, AccountProxyInterface $current_user, RouteMatchInterface $route_match) {
    parent::__construct($config_factory, $current_user, $route_match);
    $this->prlp_settings = $config_factory->get('prlp.settings');
  }

  /**
   * Event callback to check it password status table should be visible.
   */
  public function skipVisibility(CheckVisibilityEvent $event): void {

    $route_name = $this->routeMatch->getRouteName();

    if (in_array($route_name, ['user.reset.form'])) {

      $visibility_params = &$event->getParameters();

      $visibility_params['verify_email_before_password'] = FALSE;
      $visibility_params['is_route_without_password'] = FALSE;

      $uid = \Drupal::routeMatch()->getParameter('uid');
      $user = User::load($uid);
      $user_roles = $user->getRoles();
      $visibility_params['user_roles'] = $user_roles;

    }
  }

  /**
   * Event callback to check if password should be validated.
   */
  public function skipValidation(CheckValidationEvent $event): void {

    $validation_params = &$event->getParameters();

    if (!empty($this->prlp_settings)) {
      $required = $this->prlp_settings->get('password_required');
    }
    else {
      $required = FALSE;
    }

    $validation_params['verify_email_before_password'] = !$required;
    $validation_params['is_route_without_password'] = !$required;

    $uid = \Drupal::routeMatch()->getParameter('uid');
    if (!empty($uid)) {
      $user = User::load($uid);
      $user_roles = $user->getRoles();
      $validation_params['user_roles'] = $user_roles;
    }
  }

  /**
   * Event callback to validate password on reset.
   */
  public function resetPasswordValidation(ResetPasswordValidationEvent $event): void {
    $form_state = &$event->getFormState();
    $user = $event->getUser();
    $passwords = \Drupal::request()->request->all('pass');
    /** @var $validation_report \Drupal\password_policy\PasswordPolicyValidationReport */
    $validation_report = \Drupal::service('password_policy.validator')
      ->validatePassword(
        reset($passwords),
        $user,
        $user->getRoles()
      );
    if ($validation_report->hasErrors()) {
      $form_state->setErrorByName('pass2', $validation_report->getErrors());
    }
  }

  /**
   * Event callback to reset the password policy expiration data.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return void
   */
  function resetPasswordUpdate(ResetPasswordUpdateEvent $event) {
    $user = &$event->getUser();
    $date = \Drupal::service('date.formatter')->format(\Drupal::time()->getRequestTime(), 'custom', DateTimeItemInterface::DATETIME_STORAGE_FORMAT, DateTimeItemInterface::STORAGE_TIMEZONE);
    $user->set('field_last_password_reset', $date);
    $user->set('field_password_expiration', '0');
    $user->set('field_pending_expire_sent', '0');
  }

  /**
   * Registers the methods in this class that should be listeners.
   *
   * @return array
   *   An array of event listener definitions.
   */
  public static function getSubscribedEvents(): array {
    $events[PasswordPolicyExtrasEvents::CHECK_VISIBILITY][] =
      ['skipVisibility', 900];
    $events[PasswordPolicyExtrasEvents::CHECK_VALIDATION][] =
      ['skipValidation', 900];
    $events[PrlpEvents::RESET_PASSWORD_VALIDATION][] =
      ['resetPasswordValidation', 800];
    $events[PrlpEvents::RESET_PASSWORD_UPDATE][] =
      ['resetPasswordUpdate', 800];
    return $events;
  }

}
